// Class Header
class SunshineAutos {

	// The Constructor.
 	constructor( ... ) {

		// Properties -	we're using 'vargv' to allow the addition of new properties in future.
		VehicleID		=	vargv[ 0 ];
		VehicleModel		=	vargv[ 1 ];
		VehicleCat		=	vargv[ 2 ];
		VehiclePrice		=	vargv[ 3 ];
		IsPurchased		=	vargv[ 4 ];
		IsShared		=	vargv[ 5 ];
	}

	// Our static variables.
	static MyArray = "MyVehicles";

	// Member Functions.
	function ForSale()
	{
			/*
			Method: 
			      	Creates a tempory string containing all the catagories for the vehicles that are forsale.
						Example: tmp = "car bike car van car boat car bike boat"
				we will then count the number of times each catagory occurs in our tempory string.

			*/

		local tmp = "";

		// Iterate the array, and add all the catagories for the vehicles that are forsale, to a tempory string.
		foreach ( obj in this[ MyArray ] )
		{
			// Ensure we have an instance of our class, and the IsPurchased property is set to null.
			if ( obj != null )
			{
				// Add the property value to a tmp variable, so we can access squirrel's VM built in manipulation functions for that data type.
				local val = obj.IsPurchased;

				// Check the value is not null or empty
				if ( ( val == null ) || ( val.len() == 0 ) )
				{
					// Build the string.
					tmp = tmp + " " + obj.VehicleCat;
				}
			}
		}

		// Split this string, so we can search it.
		local var = split( tmp, " " );

		// Our output string.
		local output = "";

		// Iterate the array again, this time to do a string comparison and count the number of times it matches.
		foreach ( obj in this[ MyArray ] )
		{
			if ( obj != null )
			{
				// Counter to count the vehicles for each catagory.
				local count = 0;

				// Iterate our tempory string of catagories.
				for ( local i = 0; i < var.len(); i++ )
				{
					// everytime we find the substring 'VehicleCat' in our tempory string, increase our counter.
					if ( var[ i ].find( obj.VehicleCat ) != null ) count++;
				}

				// Finally, add the catagory and the count to our output, if the catagory does not exist already.
				if ( !( output.find( obj.VehicleCat ) ) ) output = output + " [ " + obj.VehicleCat + ": " + count + " ] ";
			}
		}


		// return output, if any.
		return tmp != "" ? output : "No Vehicles ForSale";
	}

	function MyCars(player)
	{
		// Empty string for our output.
		local output = "";
 
		// Iterate the array, using a foreach loop.
		foreach ( obj in this[ MyArray ] )
		{
			if ( obj != null )
			{
				// Add the object properties to a tmp variable, so we can access squirrel's VM built in manipulation functions for that data type.
				local tmp = obj.IsPurchased;

				// Check the value is not null or empty
				if ( ( tmp != null ) && ( tmp.len() > 0 ) )
				{
					// Perform a string comparison, if we get a match, add it to our output string.
					if ( tmp.find( player ) != null ) output = output + " [ " + obj.VehicleModel + ": " + obj.VehicleID + " ] "
				}
			}
		}

		// return output, if it is not still an empty string.
		return output != "" ? output : "You have not purchased a vehicle.";
	}

	function SharedCars(player)
	{
		// Empty string for our output.
		local output = "";
 
		// Iterate the array, using a foreach loop.
		foreach ( obj in this[ MyArray ] )
		{
			if ( obj != null )
			{
				// Add the object properties to a tmp variable, so we can access squirrel's VM built in manipulation functions for that data type.
				local tmp = obj.IsShared;

				// Check the value is not null or empty
				if ( ( tmp != null ) && ( tmp.len() > 0 ) )
				{
					// Perform a string comparison, if we get a match, add it to our output string.
					if ( tmp.find( player ) != null ) output = output + " [ " + obj.VehicleModel + ": " + obj.VehicleID + " ] "
				}
			}
		}

		// return output, if it is not still an empty string.
		return output != "" ? output : "You are not sharing any vehicles.";
	}

	function LoadDatabase(database)
	{
		// Create/load the database
 		db <- ::ConnectSQL( database );

		// Create a dynamic array, using our static variable as its identifer.
		this[ MyArray ] <- [];

		// Insert a null value into index zero. ( because Vehicle ID's start at 1 )
		this[ MyArray ].insert( 0, null );

		// Query the database
		local q = ::QuerySQL( db, "SELECT * FROM '" + MyArray + "'"  );

		// int to count rows
		local n = 0;

		// Iterate the rows in a loop until there are no more rows (column data = null)
		while ( ::GetSQLColumnData(q, 0 ) != null )
		{
			local
                    	    VehicleID		=    	::GetSQLColumnData( q, 0 ),
			    VehicleModel	=       ::GetSQLColumnData( q, 1 ),
			    VehicleCat		=	::GetSQLColumnData( q, 2 ),
			    VehiclePrice	=	::GetSQLColumnData( q, 3 ),
			    IsPurchased		=	::GetSQLColumnData( q, 4 ),
			    IsShared		=	::GetSQLColumnData( q, 5 );

			// Create a SunshineAutos instance for each vehicle in the database,
			// push it into the next array element and set's it's properties.
			this[ MyArray ].push( SunshineAutos( VehicleID, VehicleModel, VehicleCat, VehiclePrice, IsPurchased, IsShared ) );

     			// Read the next row available
     			::GetSQLNextRow( q );

			// Increase row count
	   		n++;
		}

		// Close the database, we're not using it anymore.
		try { ::FreeSQLQuery( q ) } catch( e ) { print( e ) }

		// Now, if it exists, Create a pointer to our class array. So we can access it outside the class scope.
		try { ::MyVehicles <- this[ MyArray ].weakref() } catch( e ) { print( e ) }
	}

	function SaveDatabase(database)
	{
		// Iterate the array
		foreach ( obj in this[ MyArray ] )
		{
			// Check we have an actual instance in the array element.
			if ( obj != null ) 
			{
				// Instance properties.
				local
                    	    	    pVehicleID		=    	obj.VehicleID,
			    	    pVehicleModel	=    	obj.VehicleModel,
			    	    pVehicleCat		=	obj.VehicleCat,
			    	    pVehiclePrice	=	obj.VehiclePrice,
			    	    pIsPurchased	=	obj.IsPurchased != null ? obj.IsPurchased : "",
			    	    pIsShared		=	obj.IsShared 	!= null ? obj.IsShared 	  : "";
			
				// The SQL Query we wish to execute.
				local query = @" UPDATE " 		+ 		MyArray 	+

				   	       " SET 	 VehicleModel=" + 	"'" + 	pVehicleModel 	+ "'"	+ ", " +
						    	"VehicleCat=" 	+ 	"'" + 	pVehicleCat 	+ "'" 	+ ", " +
						    	"VehiclePrice="	+	"'" + 	pVehiclePrice 	+ "'" 	+ ", " +
						   	"IsPurchased="	+ 	"'" + 	pIsPurchased 	+ "'" 	+ ", " +
						    	"IsShared="	+ 	"'" + 	pIsShared 	+ "'"	+

				   	       	" WHERE VehicleID=" 	+ 	pVehicleID;

       				// Execute the query
          			::QuerySQL( db, query );

				// Free it, thats the last time we need it
				// ::FreeSQLQuery( query );
			}
		}
	}

	// Reset Property values
	VehicleID		=	null;
	VehicleModel		=	null;
	VehicleCat		=	null;
	VehiclePrice		=	null;
	IsPurchased		=	null;
	IsShared		=	null;
}